/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/Char.hpp
 *  \brief  Definition of the type Char and related operators.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.3 $
 *  $Date: 2004/04/25 23:12:20 $
 */

#ifndef Beagle_Char_hpp
#define Beagle_Char_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/WrapperT.hpp"


namespace Beagle {

/*!
 *  \brief Beagle wrapper of the atomic char type.
 *  \ingroup Adapt
 */
typedef WrapperT< char >
        Char;

}


/*!
 *  \brief     Test whether an char is less than another.
 *  \param     inLeftChar  Left char compared.
 *  \param     inRightChar Right char compared.
 *  \return    True if left char is less than the right one, false if not.
 *  \par Note:
 *    The operator is defined relatively to the function isLess of Beagle::Char.
 */
inline bool operator<(const Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  return inLeftChar.isLess(inRightChar);
}


/*!
 *  \brief     Test whether an char is less than, or equal to another.
 *  \param     inLeftChar  Left char compared.
 *  \param     inRightChar Right char compared.
 *  \return    True if left char is less than, or equal to the right one, false if not.
 *  \par Note:
 *    The operator is defined relatively to the functions isLess and isEqual of Beagle::Char.
 */
inline bool operator<=(const Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  return ( inLeftChar.isLess(inRightChar) || inLeftChar.isEqual(inRightChar) );
}


/*!
 *  \brief     Test whether an char is more than another.
 *  \param     inLeftChar  Left char compared.
 *  \param     inRightChar Right char compared.
 *  \return    True if left char is more than the right one, false if not.
 *  \par Note:
 *    The operator is defined relatively to the function isLess of Beagle::Char.
 */
inline bool operator>(const Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  return inRightChar.isLess(inLeftChar);
}


/*!
 *  \brief     Test whether an char is more than, or equal to another.
 *  \param     inLeftChar  Left char compared.
 *  \param     inRightChar Right char compared.
 *  \return    True if left char is more than, or equal to the right one, false if not.
 *  \par Note:
 *    The operator is defined relatively to the functions isLess and isEqual of Beagle::Char.
 */
inline bool operator>=(const Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  return ( inRightChar.isLess(inLeftChar) || inLeftChar.isEqual(inRightChar) );
}


/*!
 *  \brief     Test whether an char is equal to another.
 *  \param     inLeftChar  Left char compared.
 *  \param     inRightChar Right char compared.
 *  \return    True if left char is equal to the right one, false if not.
 *  \par Note:
 *    The operator is defined relatively to the function isEqual of Beagle::Char.
 */
inline bool operator==(const Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  return inLeftChar.isEqual(inRightChar);
}


/*!
 *  \brief     Test whether an char is not equal to another.
 *  \param     inLeftChar  Left char compared.
 *  \param     inRightChar Right char compared.
 *  \return    True if left char is not equal to the right one, false if it is.
 *  \par Note:
 *    The operator is defined relatively to the function isEqual of Beagle::Char.
 */
inline bool operator!=(const Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  return ( inLeftChar.isEqual(inRightChar) == false);
}


/*!
 *  \brief   Compare equality of a Char with a char.
 *  \param   inLeftChar Left value to compare.
 *  \param   inRightChar Right value to compare.
 *  \return  Result of the comparison.
 */
inline
bool operator==(const Beagle::Char& inLeftChar, char inRightChar)
{
  return inLeftChar.getWrappedValue() == inRightChar;
}


/*!
 *  \brief   Compare inequality of a Char with a char.
 *  \param   inLeftChar Left value to compare.
 *  \param   inRightChar Right value to compare.
 *  \return  Result of the comparison.
 */
inline
bool operator!=(const Beagle::Char& inLeftChar, char inRightChar)
{
  return inLeftChar.getWrappedValue() != inRightChar;
}


/*!
 *  \brief   Test if a Char is < than a char.
 *  \param   inLeftChar Left value to compare.
 *  \param   inRightChar Right value to compare.
 *  \return  Result of the comparison.
 */
inline
bool operator<(const Beagle::Char& inLeftChar, char inRightChar)
{
  return inLeftChar.getWrappedValue() < inRightChar;
}


/*!
 *  \brief   Test if a Char is <= than a char.
 *  \param   inLeftChar Left value to compare.
 *  \param   inRightChar Right value to compare.
 *  \return  Result of the comparison.
 */
inline
bool operator<=(const Beagle::Char& inLeftChar, char inRightChar)
{
  return inLeftChar.getWrappedValue() <= inRightChar;
}


/*!
 *  \brief   Test if a Char is > than a char.
 *  \param   inLeftChar Left value to compare.
 *  \param   inRightChar Right value to compare.
 *  \return  Result of the comparison.
 */
inline
bool operator>(const Beagle::Char& inLeftChar, char inRightChar)
{
  return inLeftChar.getWrappedValue() > inRightChar;
}


/*!
 *  \brief   Test if a Char is >= than a char.
 *  \param   inLeftChar Left value to compare.
 *  \param   inRightChar Right value to compare.
 *  \return  Result of the comparison.
 */
inline
bool operator>=(const Beagle::Char& inLeftChar, char inRightChar)
{
  return inLeftChar.getWrappedValue() >= inRightChar;
}


/*!
 *  \brief   Increment a Char (prefix version).
 *  \param   inChar Char to increment.
 *  \return  Char incremented.
 */
inline
Beagle::Char& operator++(Beagle::Char& inChar)
{
  inChar.getWrappedValue()++;
  return inChar;
}


/*!
 *  \brief   Increment a Char (postfix version).
 *  \param   inChar Char to increment.
 *  \return  Char before being incremented.
 */
inline
Beagle::Char operator++(Beagle::Char& inChar, int)
{
  Beagle::Char lChar = inChar;
  inChar.getWrappedValue()++;
  return lChar;
}


/*!
 *  \brief   Add two Char.
 *  \param   inLeftChar Left value to add.
 *  \param   inRightChar Right value to add.
 *  \return  Result of the addition.
 */
inline
Beagle::Char operator+(const Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  return inLeftChar.getWrappedValue() + inRightChar.getWrappedValue();
}


/*!
 *  \brief   Add a Char with a char.
 *  \param   inLeftChar Left value to add.
 *  \param   inRightChar Right value to add.
 *  \return  Result of the addition.
 */
inline
Beagle::Char operator+(const Beagle::Char& inLeftChar, char inRightChar)
{
  return inLeftChar.getWrappedValue() + inRightChar;
}


/*!
 *  \brief   Add a Char to a Char.
 *  \param   inLeftChar Left value to which the right one is added.
 *  \param   inRightChar Value to add.
 *  \return  Result of the addition.
 */
inline
Beagle::Char& operator+=(Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  inLeftChar.getWrappedValue() += inRightChar.getWrappedValue();
  return inLeftChar;
}


/*!
 *  \brief   Add a char to a Char.
 *  \param   inLeftChar Left value to which the right one is added.
 *  \param   inRightChar Value to add.
 *  \return  Result of the addition.
 */
inline
Beagle::Char& operator+=(Beagle::Char& inLeftChar, char inRightChar)
{
  inLeftChar.getWrappedValue() += inRightChar;
  return inLeftChar;
}


/*!
 *  \brief   Decrement a Char (prefix version).
 *  \param   inChar Char to decrement.
 *  \return  Char decremented.
 */
inline
Beagle::Char& operator--(Beagle::Char& inChar)
{
  inChar.getWrappedValue()--;
  return inChar;
}


/*!
 *  \brief   Decrement a Char (postfix version).
 *  \param   inChar Char to decrement.
 *  \return  Char before being decremented.
 */
inline
Beagle::Char operator--(Beagle::Char& inChar, int)
{
  Beagle::Char lChar = inChar;
  inChar.getWrappedValue()--;
  return lChar;
}



/*!
 *  \brief   Subtract two Char.
 *  \param   inLeftChar Left value to subtract.
 *  \param   inRightChar Right value to subtract.
 *  \return  Result of the subtract.
 */
inline
Beagle::Char operator-(const Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  return inLeftChar.getWrappedValue() - inRightChar.getWrappedValue();
}


/*!
 *  \brief   Subtract a char from a Char.
 *  \param   inLeftChar Left value to subtract.
 *  \param   inRightChar Right value to subtract.
 *  \return  Result of the subtract.
 */
inline
Beagle::Char operator-(const Beagle::Char& inLeftChar, char inRightChar)
{
  return inLeftChar.getWrappedValue() - inRightChar;
}


/*!
 *  \brief   Subtract a Char from a Char.
 *  \param   inLeftChar Left value from which the right char is subtracted.
 *  \param   inRightChar Value to subtract.
 *  \return  Result of the subtraction.
 */
inline
Beagle::Char& operator-=(Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  inLeftChar.getWrappedValue() -= inRightChar.getWrappedValue();
  return inLeftChar;
}


/*!
 *  \brief   Subtract a char from a Char.
 *  \param   inLeftChar Left value from which the right char is subtracted.
 *  \param   inRightChar Value to subtract.
 *  \return  Result of the subtraction.
 */
inline
Beagle::Char& operator-=(Beagle::Char& inLeftChar, char inRightChar)
{
  inLeftChar.getWrappedValue() -= inRightChar;
  return inLeftChar;
}


/*!
 *  \brief   Multiply two Char.
 *  \param   inLeftChar Left value to multiply.
 *  \param   inRightChar Right value to multiply.
 *  \return  Result of the multiply.
 */
inline
Beagle::Char operator*(const Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  return inLeftChar.getWrappedValue() * inRightChar.getWrappedValue();
}


/*!
 *  \brief   Multiply a Char with a char.
 *  \param   inLeftChar Left value to multiply.
 *  \param   inRightChar Right value to multiply.
 *  \return  Result of the multiply.
 */
inline
Beagle::Char operator*(const Beagle::Char& inLeftChar, char inRightChar)
{
  return inLeftChar.getWrappedValue() * inRightChar;
}


/*!
 *  \brief   Multiply a Char with a Char.
 *  \param   inLeftChar Left value to which the right char is multiplied.
 *  \param   inRightChar Right value to multiply.
 *  \return  Result of the multiplication.
 */
inline
Beagle::Char& operator*=(Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  inLeftChar.getWrappedValue() *= inRightChar.getWrappedValue();
  return inLeftChar;
}


/*!
 *  \brief   Multiply a Char with a char.
 *  \param   inLeftChar Left value from which the right char is multiplied.
 *  \param   inRightChar Right value to multiply.
 *  \return  Result of the multiplication.
 */
inline
Beagle::Char& operator*=(Beagle::Char& inLeftChar, char inRightChar)
{
  inLeftChar.getWrappedValue() *= inRightChar;
  return inLeftChar;
}


/*!
 *  \brief   Divide two Char.
 *  \param   inLeftChar Left value to divide.
 *  \param   inRightChar Right value to divide.
 *  \return  Result of the division.
 */
inline
Beagle::Char operator/(const Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  return inLeftChar.getWrappedValue() / inRightChar.getWrappedValue();
}


/*!
 *  \brief   Divide a Char with a char.
 *  \param   inLeftChar Left value to divide.
 *  \param   inRightChar Right value to divide.
 *  \return  Result of the division.
 */
inline
Beagle::Char operator/(const Beagle::Char& inLeftChar, char inRightChar)
{
  return inLeftChar.getWrappedValue() / inRightChar;
}


/*!
 *  \brief   Divide a Char with a Char.
 *  \param   inLeftChar Left value to which the right char is divided.
 *  \param   inRightChar Right value to divide.
 *  \return  Result of the division.
 */
inline
Beagle::Char& operator/=(Beagle::Char& inLeftChar, const Beagle::Char& inRightChar)
{
  inLeftChar.getWrappedValue() /= inRightChar.getWrappedValue();
  return inLeftChar;
}


/*!
 *  \brief   Divide a Char with a char.
 *  \param   inLeftChar Left value from which the right char is divided.
 *  \param   inRightChar Right value to divide.
 *  \return  Result of the division.
 */
inline
Beagle::Char operator/=(Beagle::Char& inLeftChar, char inRightChar)
{
  inLeftChar.getWrappedValue() /= inRightChar;
  return inLeftChar;
}


#endif // Beagle_Char_hpp



